import os
import re

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.utils.html import urlize
from django.utils.http import urlquote_plus as django_urlquote_plus

register = template.Library()


@register.simple_tag
def querydict_set(querydict, param_name, param_value):
    """
    Adds/replaces/deletes specific parameter in the querydict.

    Usage example::

        <a href="?{% querydict_set request.GET 'order' 'code' %}">
    """
    # Make it mutable.
    querydict = querydict.copy()

    # Set or delete the value.
    if force_unicode(param_value):
        querydict[param_name] = param_value
    else:
        querydict.pop(param_name, None)

    # Encode into the query string.
    return querydict.urlencode()


@register.filter
def get(mapping, key):
    """
    Gets value from mapping or None.
    Example::
        {{ choices.countries|get:item.country_id }}
    """
    return mapping.get(key)


@register.filter
def getfrom(key, mapping):
    """
    Gets value from mapping or None (reversed arguments).
    Example (need to convert the year's value to int first)::
        {{ d.0|get:year|add:0|getfrom:choices.turnover }}
    """
    return mapping.get(key)


INTSPACE_RE = re.compile(r'^(-?\d+)(\d{3})')


@register.filter(is_safe=True)
def intspace(value):
    """
    Converts an integer to a string containing spaces every three digits.
    For example, 3000 becomes '3 000' and 45000 becomes '45 000'.
    """
    orig = force_unicode(value)
    new = INTSPACE_RE.sub('\g<1> \g<2>', orig)
    return new if orig == new else intspace(new)


@register.filter(is_safe=True)
def intspace_r(value):
    """
    Like intspace, but orders groups of 1 or 2 digits to the end
    instead of the beginning.
    """
    return intspace(force_unicode(value)[::-1])[::-1]


@register.filter
def startswith(s1, s2):
    """
    Proxy to the startswith method.
    """
    return s1.startswith(s2)


@register.filter
@stringfilter
def urlizetruncblank(value, limit, autoescape=None):
    """
    Converts URLs in text into clickable links, truncating URLs to the given character
    limit, and adding 'rel=nofollow' attribute to discourage spamming.
    The target is opened in the blank browser window.

    Argument: Length to truncate URLs to.
    """
    u = urlize(value, trim_url_limit=int(limit), nofollow=True, autoescape=autoescape)
    u = u.replace('<a ', '<a target="_blank" ')
    return mark_safe(u)
urlizetruncblank.is_safe = True
urlizetruncblank.needs_autoescape = True


@register.filter(is_safe=False)
@stringfilter
def urlquote_plus(value, safe=None):
    """
    Escapes a value for use in a URL, but also replaces spaces by plus signs, as required
    for quoting HTML form values when building up a query string to go into a URL.
    This is a _plus version of the standard Django urlencode defaultfilter.
    """
    kwargs = {}
    if safe is not None:
        kwargs['safe'] = safe
    return django_urlquote_plus(value, **kwargs)


@register.filter
@stringfilter
def middle_truncate(value, size):
    """
    Truncates a string to the given size placing the ellipsis in the middle.
    """
    size = int(size)
    if len(value) > size:
        if size > 3:
            start = (size - 3) / 2
            end = (size - 3) - start
            return value[:start] + u'\u2026' + value[-end:]
        else:
            return value[:size] + u'\u2026'
    else:
        return value


@register.filter
def file_exists(fieldfile):
    """
    Calls the file storage backend to test whether the file physically exists.
    There's no obvious way Django 1.6 offers this.

    Example given your Comment model has `attachment` FileField::

        <a href="{{ comment.attachment.url }}">
            {{ comment.attachment.name|strip_path }}
        </a>
        {% if comment.attachment|file_exists %}
            ({{ comment.attachment.size|filesizeformat }})
        {% endif %}
    """
    return fieldfile.storage.exists(fieldfile.path)


@register.filter
@stringfilter
def strip_path(filepath):
    """
    'same/path/to/filename.jpg' -> 'filename.jpg'

    For example usage see doc of file_exists.
    """
    return os.path.split(filepath)[1]



from functools import wraps
import hashlib

from django.core import cache  # Importing this way so debug_toolbar can patch it later.
from django.core.cache.backends.base import DEFAULT_TIMEOUT


ALLOWED_CACHED_FUNCTION_ARG_TYPES = set((type(None), int, float, long, bool, str, unicode))


def cached_function(func=None, num_args_to_key=None, timeout=DEFAULT_TIMEOUT):
    """
    We all write this tedious code often::

        key = make_key(arg1, arg2, arg3)
        value = cache.get(key)
        if value is None:
            value = expensive(arg1, arg2, arg3)
            cache.set(key, value, timeout=3600)

        return value

    This decorator tries to remove the need to do this. You only use the following::

        @cached_function(timeout=3600)
        def expensive(arg1, arg2, arg3):
            ...

    To use the Django default timeout, you can omit the arguments::

        @cached_function

    The expensive() must accept the fixed number of positional arguments. All these args must be
    basic and easily repr-esentable Python types, in order a solid cache key can be made from them
    (as well as from the function name and containing module name).

    If you pass arguments to expensive() that do not need to be part of the cache key, put them
    to the end of args and limit the number of key-making args from the left::

        @cached_function(num_args_to_key=2)

    """
    def cached_function_inner(func):
        if func.func_defaults:
            raise RuntimeError("cached_function does not allow function to have default args "
                               "in order to keep the number of passed args fixed.")

        if func.__name__ == '<lambda>':
            raise RuntimeError("cached_function can't work on anonymous funcs in order "
                               "to keep the name unique.")

        @wraps(func)
        def wrapper(*args):
            key_args = args if num_args_to_key is None else args[:num_args_to_key]

            unallowed_types = set(map(type, key_args)) - ALLOWED_CACHED_FUNCTION_ARG_TYPES
            if unallowed_types:
                raise RuntimeError("cached_function received unallowed types of keyable args: %s"
                                   % ', '.join(map(str, unallowed_types)))

            key_fmt = 'cached_function:%s.%s(%%s)' % (func.__module__, func.__name__)
            key = key_fmt % ','.join(map(repr, key_args))

            if len(key) > 200:
                key_args = hashlib.sha256(key_args).hexdigest()
                key = key_fmt % key_args + '-hashed'

            value = cache.cache.get(key)

            if value is None:
                value = func(*args)

                cache.cache.set(key, value, timeout=timeout)

            return value

        return wrapper

    if func is None:
        # @cached_function(...)
        return cached_function_inner

    elif callable(func):
        # @cached_function
        return cached_function_inner(func)

    else:
        raise RuntimeError("Invalid arguments provided to cached_function decorator.")



class AttrDictSetCollision(Exception):
    """
    Raised when the AttrDict refuses to change any dict's attribute.
    """


class AttrDict(dict):
    """
    This is a dict whose values can be accessed also as attributes.
    Setting attribute occuring in the original dict class is forbidden.
    """
    def __setattr__(self, name, value):
        if hasattr(dict, name):
            raise AttrDictSetCollision(name)

        self[name] = value

    __getattr__ = dict.__getitem__

    def __getstate__(self):
        return self.__dict__.copy()

    def __setstate__(self, items):
        self.__dict__.update(items)

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, dict.__repr__(self))


try:
    from collections import OrderedDict
except ImportError:
    pass
else:
    class OrderedAttrDictSetCollision(Exception):
        """
        Raised when the OrderedAttrDict refuses to change any OrderedDict's attribute.
        """

    class OrderedAttrDict(OrderedDict):
        """
        Like AttrDict, but remembers ordering in which values were set.
        """
        def __setattr__(self, name, value):
            """
            Needs a fork to allow OrderedDict to establish some private names in constructor.
            """
            if name.startswith('_OrderedDict__'):
                return super(OrderedAttrDict, self).__setattr__(name, value)

            if hasattr(OrderedDict, name):
                raise OrderedAttrDictSetCollision(name)

            self[name] = value

        def __getattr__(self, name):
            """
            While AttrDict.__getattr__ is raising KeyError, this one needs to convert it to
            AttributeError as OrderedDict establishes some names in constructor.
            """
            try:
                return self.__getitem__(name)
            except KeyError, e:
                raise AttributeError(e)



import re
from functools import wraps
try:
    from urllib.parse import urlparse
except ImportError:     # Python 2
    from urlparse import urlparse

from django.conf import settings
from django.conf.urls import url
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.core.exceptions import PermissionDenied
from django.shortcuts import resolve_url
from django.utils.decorators import available_attrs
from django.utils.encoding import force_str


class UrlPatterns(list):
    """
    Offers to create the standard urlpatterns in views.py instead of in urls.py and supports DRY
    by allowing to not reference the view name and not define the URL name (it can be
    automatically set by the name of the view func). The URL patterns are then closer to the views.

    IMPORTANT::

        As it appends the view to the URLConf, this is the last (outermost) decorator
        able to change the view behavior!

    Example::

        ### views.py (your exams/urls.py is no more)

        from useful.django.urlpatterns import UrlPatterns

        urlpatterns = UrlPatterns()

        @urlpatterns.url(r'^list/$')
        def exams_list(request):
            ...

    In project *root* urls.py::

        url(r'^exams/', include('myproject.apps.exams.views')),

    Now /exams/list/ url lives under the name exams_list as expected::

        {% url 'exams_list' %}

    Permissions:

    Also this class supports unification in permission definition and reference.

    Until now, every time you wanted to test whether the user can see the link to the view,
    you had to test for the same set of conditions the view was decorated with.

    That is not very DRY.

    `can_url' function and `can_url_processor' offer to define even the complex set of permissions
    only once.  All references only use the view spec usually used in the {% url %} tag.

    Example::

        ### models.py
        ...
        class Meta:
            verbose_name = _("Exam")
            verbose_name_plural = _("Exams")
            permissions = (
                ('see_listing', "Can see the listing of exams"),
            )

        ### views.py

        @urlpatterns.url(r'^list/$', perms='exams.see_listing')
        def exams_list(request):
            ...

    Now in template, to *test whether the user can access the view*::

        ### settings.py

        TEMPLATE_CONTEXT_PROCESSORS += (
            ...
            'useful.django.can_url.can_url_processor',
        )

        ### in some template.html

        {% if can_url.exams_list %}
            <a href="{% url 'exams_list' %}">...</a>
        {% endif %}

    Alternatively as dict: {% if 'namespace:exams_list' in can_url %} ...

    Allowed forms of perm definition in the decorator::

        perms='PERM1'                     # the permission must be present
        perms=('PERM1', 'PERM2')          # all these perms must be present
        perms='PERM1 | PERM2'             # user needs PERM1 or PERM2
        perms=('PERM1', 'PERM2 | PERM3')  # PERM1 and (PERM2 or PERM3)

    Each of PERM1, PERM2, PERM3 can be:

        'is_active'
        'is_authenticated'
        'is_staff'
        'is_superuser'
            - these are tests for appropriate attribute of the user

        'app_name.perm_name'
            - if the dot ('.') is present, user.has_perm() is called to test

        Otherwise user.has_module_perms() is called (that usually tests whether
        the user has any permission for the given app).

    Consider these permissions::

        ### models.py
        ...
        class Meta:
            permissions = (
                ('see_own_exams',  "Can SEE OWN submitted exams only"),
                ('see_all_exams',  "Can SEE ALL submitted exams"),
            )

    Each PERM can also be followed by its alias in parentheses::

        ### views.py

        @urlpatterns.url(r'^list/$', perms='exams.see_own_exams | exams.see_all_exams (can_all)')
        def exams_list(request, can_all=False):
            ...
            exams = Exam.objects.all()

            if can_all and request.GET.get('showall'):
                messages.info(request, _("The list of exams of all users."))
            else:
                exams = exams.filter(user=request.user)
                messages.info(request, _("The list of your past exams."))
            ...
            context['can_all'] = can_all  # can be useful in rendered template

    It is required for the view to accept an optional argument can_all that
    is True only when possessing the exams.see_all_exams and can be helpful
    in further DRYing the view behavior.

    It's also easy to indirectly test for permission in Python like you do in template::

        from useful.django.can_url import can_url

        return HttpResponseRedirect(reverse('exams_list') if can_url(request.user, 'exams_list')
                                    else '/')

    """
    # Note: Python 3 will allow all Unicode letters and digits in its identifiers.
    PERM_RE = r'\s*([a-zA-Z_][\w.]*)\s*(\(\s*([a-zA-Z_]\w*)\s*\))?\s*'

    def __init__(self, login_url=None, raise_exception=False, redirect_field_name=None):
        self.login_url = login_url
        self.raise_exception = raise_exception
        self.redirect_field_name = redirect_field_name or REDIRECT_FIELD_NAME

    def url(self, regex, kwargs=None, name=(), perms=None):
        def decorator(view_func):
            if perms is None:
                _wrapped = view_func
            else:
                compiled_anded_perms = self.compile_perms(perms)

                def __can_url(user):
                    return self._can_url(compiled_anded_perms, user)

                _wrapped = self.user_passes_test(__can_url)(view_func)
                _wrapped.can_url_func = __can_url
                _wrapped.can_url_perms = perms
                _wrapped.can_url_perms_compiled = compiled_anded_perms

            # Assigning different name to avoid changing the nonlocal variable.
            url_name = view_func.func_name if name is () else name
            self.append(url(regex, _wrapped, kwargs=kwargs, name=url_name))

            return _wrapped

        return decorator

    def compile_perms(self, perms):
        """
        Accepts the permission definition, returns list of AND-ed tuples.
        Each tuple contains OR-ed pairs (single-perm, alias-value).
        Alias-value is either text from parentheses or True.
        """
        if not isinstance(perms, (list, tuple)):
            perms = (perms,)

        def comp(ored_perms):
            for permdef in ored_perms.split('|'):
                m = re.match(self.PERM_RE, permdef)
                if not m:
                    raise RuntimeError("Cannot parse perm definition: %r" % permdef)
                perm, unused, alias = m.groups()
                yield perm, alias or True

        return [tuple(comp(ored_perms)) for ored_perms in perms]

    def user_passes_test(self, test_func):
        """
        Decorator based on django.contrib.auth.decorators.user_passes_test.
        Extended with passing the resolved aliased permissions to the view.
        """
        def decorator(view_func):
            @wraps(view_func, assigned=available_attrs(view_func))
            def _wrapped_view(request, *args, **kwargs):
                perm_aliases = test_func(request.user)
                if perm_aliases:
                    perm_aliases.discard(True)
                    for alias in perm_aliases:
                        assert alias not in kwargs
                        kwargs[alias] = True
                    return view_func(request, *args, **kwargs)

                # THE FOLLOWING code behaves the same as original:

                path = request.build_absolute_uri()
                # urlparse chokes on lazy objects in Python 3, force to str
                resolved_login_url = force_str(resolve_url(self.login_url or settings.LOGIN_URL))
                # If the login url is the same scheme and net location then just
                # use the path as the "next" url.
                login_scheme, login_netloc = urlparse(resolved_login_url)[:2]
                current_scheme, current_netloc = urlparse(path)[:2]
                if ((not login_scheme or login_scheme == current_scheme) and
                   (not login_netloc or login_netloc == current_netloc)):
                    path = request.get_full_path()
                from django.contrib.auth.views import redirect_to_login
                return redirect_to_login(path, resolved_login_url,
                                         self.redirect_field_name)
            return _wrapped_view
        return decorator

    def _can_url(self, compiled_anded_perms, user):
        res = set()
        for ored_perms in compiled_anded_perms:
            res2 = set(alias for perm, alias in ored_perms if self.test_perm(perm, user))
            if not res2:
                if self.raise_exception:
                    raise PermissionDenied
                return set()
            res |= res2
        return res

    def test_perm(self, perm, user):
        if perm == 'is_active':
            return user.is_active
        if perm == 'is_authenticated':
            return user.is_authenticated()
        if perm == 'is_staff':
            return user.is_staff
        if perm == 'is_superuser':
            return user.is_superuser

        # From now on, active && superuser users have all perms.

        if '.' in perm:
            return user.has_perm(perm)

        return user.has_module_perms(perm)



# Based on commit 111397cd41db41b2acec84473649b6f291a9c272 of
# https://github.com/ui/django-cached_authentication_middleware/
# Replaced User with UserModel to support the Django 1.5 swappable model.

"""
Copyright (c) 2012 Selwin Ong

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from django.conf import settings
from django.contrib.auth import get_user, SESSION_KEY
from django.core import cache  # Importing this way so debug_toolbar can patch it later.
from django.db import models
from django.db.models.signals import post_save, post_delete
from django.utils.functional import SimpleLazyObject

# Django 1.5 swappable model support, backward compatible.
try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User as UserModel
else:
    UserModel = get_user_model()

CACHE_KEY = 'cached_auth_middleware_1.5:%s'

try:
    app_label, model_name = settings.AUTH_PROFILE_MODULE.split('.')
    profile_model = models.get_model(app_label, model_name)
except (ValueError, AttributeError):
    profile_model = None


def invalidate_cache(sender, instance, **kwargs):  # @UnusedVariable
    if isinstance(instance, UserModel):
        key = CACHE_KEY % instance.id
    else:
        key = CACHE_KEY % instance.user_id

    cache.cache.delete(key)


def get_cached_user(request):
    from django.contrib.auth.models import AnonymousUser

    if not hasattr(request, '_cached_user'):
        try:
            key = CACHE_KEY % request.session[SESSION_KEY]
            user = cache.cache.get(key)
        except KeyError:
            user = AnonymousUser()

        if user is None:
            user = get_user(request)

            # Try to populate profile cache if profile is installed
            if profile_model:
                try:
                    user.get_profile()
                # Handle exception for user with no profile and AnonymousUser
                except (profile_model.DoesNotExist, AttributeError):
                    pass
            cache.cache.set(key, user)
        request._cached_user = user
    return request._cached_user


class CachedAuthenticationMiddleware(object):
    """
    A drop-in replacement for django.contrib.auth's built-in
    AuthenticationMiddleware. It tries to populate request.user by fetching
    user and profile data from cache before falling back to the database.

    'django.contrib.auth.middleware.AuthenticationMiddleware' in
    settings.MIDDLEWARE_CLASSES can be replaced with
    'useful.django.cached_auth.CachedAuthenticationMiddleware'.
    """
    def __init__(self):
        post_save.connect(invalidate_cache, sender=UserModel)
        post_delete.connect(invalidate_cache, sender=UserModel)
        if profile_model:
            post_save.connect(invalidate_cache, sender=profile_model)
            post_delete.connect(invalidate_cache, sender=profile_model)

    def process_request(self, request):
        assert hasattr(request, 'session'), \
            "The Django authentication middleware requires session middleware to be installed. "\
            "Edit your MIDDLEWARE_CLASSES setting to insert 'django.contrib.sessions.middleware.SessionMiddleware'."

        request.user = SimpleLazyObject(lambda: get_cached_user(request))



import unicodedata


GUESS_ENCODINGS_SEQUENCE = 'UTF-8', 'CP1250', 'ISO-8859-2'


def guess_decode(s, fallback_method='replace'):
    for enc in GUESS_ENCODINGS_SEQUENCE:
        try:
            return s.decode(enc)
        except UnicodeDecodeError:
            pass
    return s.decode('UTF-8', fallback_method)


def strip_accents(unistr):
    """
    Return the accent-stripped str for the given unistr (unicode is required).
    """
    return unicodedata.normalize('NFKD', unistr).encode('ascii', 'ignore')


def encoded_stream(stream, into='UTF-8'):
    import codecs

    if (stream.encoding or '').upper() != into.upper():
        stream = codecs.getwriter(into)(stream)

    return stream


def auto_encode_stdout_stderr(into='UTF-8'):
    """
    Call this on the top of your program to avoid the need of encoding each printed
    unicode object when the output encoding is not known such as during shell redirection.
    """
    import sys

    sys.stdout = encoded_stream(sys.stdout, into)
    sys.stderr = encoded_stream(sys.stderr, into)



from django.utils.translation import ugettext_lazy as _
from django.utils.text import get_text_list
from django.utils.encoding import force_unicode
from django.contrib.contenttypes.models import ContentType
from django.contrib.admin.models import LogEntry, ADDITION, CHANGE, DELETION
from django.contrib import admin
from django.template.defaultfilters import capfirst

# Django 1.5 swappable model support, backward compatible.
try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User as UserModel
else:
    UserModel = get_user_model()

from .readonly_admin import ReadOnlyModelAdmin


class UILogEntry(object):
    def __init__(self, user=None):
        """
        This is similar to how ModelAdmin logs events. Expects the user
        doing the change as argument. When not given, 'admin' is used.
        """
        if user is None:
            user = UserModel.objects.only('id').get(username='admin')
        self.user = user

    def log_addition(self, object_, message='UI'):
        """
        Adds a LogEntry recording that the object_ was added.
        The change detail message defaults to 'UI' to denote the change
        did not originate from admin interface.
        """
        self.do_log(object_, ADDITION, message)

    def log_change(self, object_, message='UI', form=None):
        """
        Adds a LogEntry recording that the object_ was changed.
        If clean form given, the list of changed fields is added to message.
        The change detail message defaults to 'UI' to denote the change
        did not originate from admin interface.
        """
        if form is not None:
            if message:
                message += ' '
            message += self.construct_change_message(form)

        self.do_log(object_, CHANGE, message)

    def log_deletion(self, object_, message='UI'):
        """
        Adds a LogEntry recording that the object_ was deleted.
        The change detail message defaults to 'UI' to denote the change
        did not originate from admin interface.
        """
        self.do_log(object_, DELETION, message)

    def construct_change_message(self, form):
        change_message = []
        if form.changed_data:
            lst = get_text_list(form.changed_data, _('and'))
            change_message.append(_('Changed %s.') % lst)

        change_message = u' '.join(change_message)
        return unicode(change_message or _('No fields changed.'))

    def do_log(self, object_, action_flag, message):
        LogEntry.objects.log_action(
            user_id=self.user.pk,
            content_type_id=ContentType.objects.get_for_model(object_).pk,
            object_id=object_.pk,
            object_repr=force_unicode(object_),
            action_flag=action_flag,
            change_message=unicode(message))

    def save_form_instance_and_log(self, form):
        """
        Shortcut to saving a clean ModelForm instance + logging
        the addition or deletion.
        """
        existed = form.instance.id

        instance = form.save()

        if existed:
            self.log_change(form.instance, form=form)
        else:
            self.log_addition(form.instance)

        return instance


CHTYPES = {admin.models.ADDITION: 'addlink',
           admin.models.CHANGE: 'changelink',
           admin.models.DELETION: 'deletelink'}


class LogEntryAdmin(ReadOnlyModelAdmin):
    """
    This read only ModelAdmin can be presented to the staff users for
    reference.
    Example::

        from django.contrib import admin
        from django.contrib.admin.models import LogEntry
        from useful.django.logentry import LogEntryAdmin

        admin.site.register(LogEntry, LogEntryAdmin)
    """
    list_display = 'action_time_nolink', 'user', 'table', 'action', 'change_message'
    list_filter = 'user__is_superuser', 'content_type'
    date_hierarchy = 'action_time'

    def action_time_nolink(self, obj):
        return '</a><span style="font-weight: normal">%s</span><a>' \
            % obj.action_time.strftime('%Y-%m-%d %H:%M:%S')
    action_time_nolink.allow_tags = True
    action_time_nolink.short_description = _("Action time")

    def table(self, obj):
        if obj.content_type_id:
            m = obj.content_type.model_class()._meta
            return capfirst(m.verbose_name_plural)
        return ''
    table.short_description = _("Table")

    def action(self, obj):
        r = obj.object_repr
        if not obj.is_deletion() and obj.get_admin_url():
            r = '<a href="%s">%s</a>' % (obj.get_admin_url(), r)
        return '<span class="%s">%s</span>' % (CHTYPES[obj.action_flag], r)
    action.allow_tags = True
    action.short_description = _("Action, object")

    def queryset(self, request):
        qs = super(LogEntryAdmin, self).queryset(request)
        return qs.select_related('user', 'content_type')



import os
import stat
import posixpath
import urllib

from django import template
from django.conf import settings
from django.contrib.staticfiles import finders, storage

register = template.Library()

STATIC_URL_CACHE = {}


@register.simple_tag
def freshstatic(path):
    """
    A template tag that returns the URL to a file using staticfiles storage
    backend.

    Example::

        <link rel="stylesheet" type="text/css" href="{% freshstatic "css/reset.css" %}" />

    Note: The staticfiles finder is expected to find the file on which
    os.stat can be run.

    The last modification time is then appended to the resulting URL.
    This has the advantage that the HTTP Expires header can be set to infinite
    and the browsers get different URL each time the file changes.

    Note 2: As the URLs here are cached on the per-process level, it is
    required the Django application is restarted in production each time
    any static file changes the content.
    """
    url = STATIC_URL_CACHE.get(path)
    if url is None:
        # Call the same method the original staticfiles 'static' tag does.
        url = storage.staticfiles_storage.url(path)

        # Get the real file abs path and its timestamp.
        normalized_path = posixpath.normpath(urllib.unquote(path)).lstrip('/')
        absolute_path = finders.find(normalized_path)

        if absolute_path:
            modtime = os.stat(absolute_path)[stat.ST_MTIME]
            url += '?%d' % modtime

        elif not getattr(settings, 'FRESHSTATIC_CAN_BE_MISSING', None):
            raise RuntimeError("Static file %s not found." % path)

        if not settings.DEBUG:
            # Save to cache (only when not developing).
            STATIC_URL_CACHE[path] = url

    return url



from functools import wraps
import os
import time
import errno

# Based on http://www.evanfosmark.com/2009/01/cross-platform-file-locking-support-in-python/


class FileLockTimeoutException(Exception):
    pass


class FileLock(object):
    """ A file locking mechanism that has context-manager support so
        you can use it in a with statement. This should be relatively cross
        compatible as it doesn't rely on msvcrt or fcntl for the locking.
        Timeout when waiting for lock is supported.

        WARNING: The code is not safe for all NFS implementations.

        See the O_EXCL section of the "open" manpage, test it first!

        Usage example::

            with FileLock('/tmp/myproject-critical-processing'):
                print "This section will only be executed by a single thread at the same time."

        You can also use the instance as the decorator::

            @FileLock('/tmp/func1-critical-processing')
            def func1():
                print "This function will only be executed by a single thread at the same time."

        Also the lock stealing is implemented (stealing=True), but only on the POSIX system where it is possible to send
        signal 0 to process to test its existence. If requested, the locking process will write a small JSON info to the
        lockfile. Each time before waiting for lock to be released, the competing process signals the lock originator
        by PID. If the locking process does not exist, the lock file is removed and acquiring is tried again.
    """
    def __init__(self, file_name, timeout=30, delay=0.2, stealing=False):
        """ Prepare the file locker. Specify the file to lock and optionally
            the maximum timeout and the delay between each attempt to lock.
        """
        self.is_locked = False
        self.lockfile = os.path.join(os.getcwd(), "%s.lock" % file_name)
        self.file_name = file_name
        self.timeout = timeout
        self.delay = delay
        self.stealing = stealing

        if stealing:
            if os.name != 'posix':
                raise RuntimeError("Detecting a running process by its PID is only supported on a POSIX system.")

            import json
            self.json = json

    def acquire(self):
        """ Acquire the lock, if possible. If the lock is in use, it check again
            every `wait` seconds. It does this until it either gets the lock or
            exceeds `timeout` number of seconds, in which case it throws
            an exception.
        """
        start_time = time.time()
        while True:
            try:
                self.fd = os.open(self.lockfile, os.O_CREAT | os.O_EXCL | os.O_RDWR)
                break
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise

                if self.should_steal():
                    os.unlink(self.lockfile)
                    continue

                if (time.time() - start_time) >= self.timeout:
                    raise FileLockTimeoutException("%d seconds passed." % self.timeout)

                time.sleep(self.delay)

        self.is_locked = True

        if self.stealing:
            import datetime
            import sys

            info = {
                'lock_time': datetime.datetime.now().isoformat(),  # warning: timezone unaware!
                'pid': os.getpid(),
                'argv': sys.argv,
            }
            os.write(self.fd, self.json.dumps(info, indent=4))
            os.fsync(self.fd)

    def release(self):
        """ Get rid of the lock by deleting the lockfile.
            When working in a `with` statement, this gets automatically
            called at the end.
        """
        if self.is_locked:
            os.close(self.fd)
            os.unlink(self.lockfile)
            self.is_locked = False

    def should_steal(self):
        if self.stealing:
            try:
                info = self.json.load(open(self.lockfile))
            except ValueError:
                # We don't steal if the lockfile is empty of contains an invalid JSON value.
                pass
            except IOError, e:
                # We ignore when the lockfile was removed between the exclusive open and load.
                if e.errno != errno.ENOENT:
                    raise
            else:
                try:
                    # POSIX test whether the given PID exists
                    os.kill(info['pid'], 0)
                except OSError as e:
                    # For EPERM we intentionally don't steal as the locking process might have higher privilege.
                    # ESRCH means "No such process".
                    if e.errno == errno.ESRCH:
                        return True

        return False

    def __enter__(self):
        """ Activated when used in the with statement.
            Should automatically acquire a lock to be used in the with block.
        """
        if not self.is_locked:
            self.acquire()
        return self

    def __exit__(self, exc_type, exc_value, traceback):  # @UnusedVariable
        """ Activated at the end of the with statement.
            It automatically releases the lock if it isn't locked.
        """
        if self.is_locked:
            self.release()

    def __del__(self):
        """ Make sure that the FileLock instance doesn't leave a lockfile
            lying around.
        """
        self.release()

    def __call__(self, function):
        """
        Support for using the instance as decorator. The entire function will be protected.
        """
        @wraps(function)
        def inner(*args, **kwargs):
            with self:
                return function(*args, **kwargs)

        return inner



import os
import time
import functools

from django.conf import settings
from django.core.paginator import Paginator, EmptyPage, InvalidPage
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseBadRequest
from django.template import RequestContext

from .crypto import SecretTokenGenerator


class HttpResponseException(Exception):
    """
    An abstract exception allowing any kind of exception under page decorator to be converted
    to a HttpResponse<Anything>.
    Don't raise this directly, but subclass and raise with http_response attribute.
    """


class ReverseAndRedirect(HttpResponseException):
    """
    A shortcut allowing to do a reverse, then redirect the user by raising an exception.
    """
    def __init__(self, viewname, *args, **kwargs):
        url = reverse(viewname, args=args, kwargs=kwargs)
        self.http_response = HttpResponseRedirect(url)


def page(template=None, **decorator_args):
    """
    Decorator to make template rendered by Django views dead simple.
    Takes the template path as first argument. See the code comments.
    Example::

        @page('payments/payments_info.html')
        def payment_info(request):
            return { ... context dict ... }
    """
    def page_decorator_wrapper(fn):
        @functools.wraps(fn)
        def page_decorator_inner_wrapper(request, *args, **kw):
            # Take the basic context dictionary from the optional decorator args.
            data = decorator_args.copy()

            try:
                # Call the original view.
                d = fn(request, *args, **kw)
            except HttpResponseException, e:
                return e.http_response

            # Return now if it returned some kind of HTTP response itself, no job.
            if isinstance(d, HttpResponse):
                return d

            if d:
                data.update(d)

            # The view can override the template to use.
            template_name = data.get('template',  template)

            # By adding the debug_template parameter we switch to possible
            # debugging template:
            # payments/payments_info.html -> payments/payments_info_debug.html
            if settings.DEBUG and request.GET.get('debug_template'):
                stn = os.path.splitext(template_name)
                template_name = stn[0] + '_debug' + stn[1]

            # The view or the decorator call can override the context
            # instance. Otherwise, use the usual RequestContext.
            context_instance = data.get('context') or RequestContext(request)

            # Render the template.
            response = render_to_response(template_name, data, context_instance)
            return response

        return page_decorator_inner_wrapper
    return page_decorator_wrapper


class JsonResponse(HttpResponse):
    """
    Returns JSON encoded object as HTTP response.
    """
    def __init__(self, response_data, indent=2, ensure_ascii=False, **kwargs):
        import json

        kwargs['content'] = json.dumps(response_data, indent=indent, ensure_ascii=ensure_ascii)
        kwargs['content_type'] = 'application/json; charset=UTF-8'
        super(JsonResponse, self).__init__(**kwargs)


def protected_redirect(request):
    """
    Redirects to the URL in GET parameter u, but only if protection check passes.
    The 'u' GET parameter should be the target URL. The 't' should be the
    token created by SecretTokenGenerator().check_token(u).
    When the token check passes, the refreshing style redirection HTML page
    is returned. This is useful in hiding the true URL from where the link
    was clicked: The target site will get the redirector URL as referrer.
    """
    u, t = request.GET.get('u'), request.GET.get('t')

    if u and t and SecretTokenGenerator().check_token(u, t):
        u = ('http://' + u) if '://' not in u else u

        return HttpResponse('''<html><head>
<meta http-equiv="refresh" content="1;url=%s">
</head>
<body>
    <a href="%s">Redirecting to %s</a>
    <script>window.location.replace("%s");
    </script>
</body></html>''' % (u, u, u, u))
    else:
        return HttpResponseBadRequest("Bad parameters or protection fault.")


def serve_with_Expires(request, path, cache_timeout=365 * 24 * 60 * 60):
    """
    This view can be used in the development server to add the Expires
    header on the static files protected by the modtime change detection
    in the freshstatic template tag (see there).

    The browsers will stop asking for the file again and again during the
    normal browsing.

    Example in your urls.py::

        urlpatterns += static.static(settings.STATIC_URL, serve_with_Expires)

    By adding a cache_timeout parameter to the above call with the number
    of seconds you will change the deafult expiration of 1 year.
    """
    from django.utils.http import http_date
    from django.contrib.staticfiles.views import serve

    response = serve(request, path)

    if not response.has_header('Expires'):
        response['Expires'] = http_date(time.time() + cache_timeout)

    return response


def paginate(request, objects, per_page=20):
    """
    This is so common Django pagination code...
    Can be used like this in the template::

        {% load i18n useful %}
        <div class="paginate">
            <ul>
                {% if objs.has_previous %}
                    <li class="first">
                        <a href="?{% querydict_set request.GET 'page' 1 %}" title="{% trans "First page" %}">
                            &lt;&lt;
                        </a>
                    </li>
                    <li>
                        <a href="?{% querydict_set request.GET 'page' objs.previous_page_number %}"
                            title="{% trans "Previous page" %}">
                            &lt;
                        </a>
                    </li>
                {% endif %}

                {% for page in objs.paginator.page_range %}
                    <li {% ifequal objs.number page %}class="active"{% endifequal %}>
                        <a href="?{% querydict_set request.GET 'page' page %}" title="{% trans "Jump to page">
                            {{ page }}
                        </a>
                    </li>
                {% endfor %}

                <span class="current">
                    {% blocktrans with objs.number as num and objs.paginator.num_pages as total %}
                        Page {{ num }} of {{ total }}
                    {% endblocktrans %}
                </span>

                {% if objs.has_next %}
                    <li>
                        <a href="?{% querydict_set request.GET 'page' objs.next_page_number %}"
                            title="{% trans "Next page" %}">
                            &gt;
                        </a>
                    </li>
                    <li class="last">
                        <a href="?{% querydict_set request.GET 'page' objs.paginator.num_pages %}"
                            title="{% trans "Last page" %}">
                            &gt;&gt;
                        </a>
                    </li>
                {% endif %}
            </ul>
        </div>
    """
    paginator = Paginator(objects, per_page)

    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1

    try:
        objects = paginator.page(page)
    except (EmptyPage, InvalidPage):
        objects = paginator.page(paginator.num_pages)

    return objects



from .getters import get_object_or_none

from django.contrib.auth.backends import ModelBackend
from django.core.validators import validate_email, ValidationError
from django.utils.crypto import get_random_string

# Django 1.5 swappable model support, backward compatible.
try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User as UserModel
else:
    UserModel = get_user_model()


def get_random_password(size=7, cadre='abcdehkmnprstuvxyz2345678923456789'):
    """
    Returns the password composed of the easily readable letters and digits
    by default (digits doubled to increase the occurrence).
    """
    return get_random_string(size, cadre)


def get_unique_username(username):
    """
    Returns the first available username with appended numbering
    (started from 2) that is not yet present in the UserModel table.
    """
    assert username
    base = username
    idx = 2
    while True:
        if not UserModel.objects.filter(username=username).exists():
            break
        username = '%s%d' % (base, idx)
        idx += 1
    return username


class EmailLoginModelBackend(ModelBackend):
    """
    Logs the user in using his/her e-mail (case insensitively by default).
    Set up by the following Django setting::

        AUTHENTICATION_BACKENDS = ('useful.django.auth.EmailLoginModelBackend',)

    """
    EMAIL_CASE_SENSITIVE = False

    def get_user_by_email(self, email):
        try:
            validate_email(email)
            if self.EMAIL_CASE_SENSITIVE:
                user = UserModel.objects.get(email__exact=email)
            else:
                user = UserModel.objects.get(email__iexact=email)
        except (UserModel.DoesNotExist, ValidationError):
            user = None

        return user

    def authenticate(self, username=None, password=None):
        user = self.get_user_by_email(username)
        return user if user and user.check_password(password) else None


class UsernameOrEmailLoginModelBackend(EmailLoginModelBackend):
    """
    Tries to identify the given login-name as username or email
    (both case-insensitive by default).
    Set up by the following Django setting::

        AUTHENTICATION_BACKENDS = ('useful.django.auth.UsernameOrEmailLoginModelBackend',)
    """
    USERNAME_CASE_SENSITIVE = False

    def authenticate(self, username=None, password=None):
        if self.USERNAME_CASE_SENSITIVE:
            user = get_object_or_none(UserModel, username__exact=username)
        else:
            user = get_object_or_none(UserModel, username__iexact=username)

        if user is None:
            user = self.get_user_by_email(username)

        return user if user and user.check_password(password) else None



from django.contrib import admin
from django.core.exceptions import PermissionDenied
from django.utils.translation import ugettext_lazy as _


class SemiReadOnlyModelAdmin(admin.ModelAdmin):
    """
    Only hides the actions and adding/deleting controls.
    """
    actions = None

    def has_add_permission(self, request, obj=None):  # @UnusedVariable
        return False
    has_delete_permission = has_add_permission
#    has_change_permission = has_add_permission


class ReadOnlyModelAdmin(SemiReadOnlyModelAdmin):
    """
    Not only hides the actions and adding/deleting controls, but really
    forbids the model change. Only change_list browsing is allowed.

    Below are few examples of first items in the list_display the remove
    the detail link.
    """
    def save_model(self, request, obj, form, change):  # @UnusedVariable - MUST NOT CHANGE THE VAR NAME
        raise PermissionDenied

    def id_nolink(self, obj):
        return '</a><span style="font-weight: normal">%s</span><a>' % obj.id
    id_nolink.allow_tags = True
    id_nolink.short_description = "ID"

    def created_nolink(self, obj):
        return '</a><span style="font-weight: normal">%s</span><a>' \
            % obj.created.strftime('%Y-%m-%d %H:%M:%S')
    created_nolink.allow_tags = True
    created_nolink.short_description = _("Created")

    def entered_nolink(self, obj):
        return '</a><span style="font-weight: normal">%s</span><a>' \
            % obj.entered.strftime('%Y-%m-%d %H:%M:%S')
    entered_nolink.allow_tags = True
    entered_nolink.short_description = _("Entered")



from django.db import connection


def truncate_table(model_or_tablename, cascade=False, cursor=None):
    if cursor is None:
        cursor = connection.cursor()

    if hasattr(model_or_tablename, '_meta'):
        tablename = model_or_tablename._meta.db_table
    else:
        tablename = model_or_tablename

    if cascade:
        stmt = 'TRUNCATE %s CASCADE'
    else:
        stmt = 'TRUNCATE %s'

    cursor.execute(stmt % connection.ops.quote_name(tablename))



import bisect
import hashlib


class ConsistentHashRing(object):
    """
    Spread the keys into a fixed set of buckets as uniformly as possible.
    Also maintain consistency so only a few keys change their assignment when the set of buckets change.

    See also https://en.wikipedia.org/wiki/Consistent_hashing
    Used code from http://techspot.zzzeek.org/2012/07/07/the-absolutely-simplest-consistent-hashing-example/
    """
    def __init__(self, buckets=None, replicas=1000):
        self.replicas = replicas
        self.keys = []
        self.bucket_map = {}

        if buckets:
            map(self.add_bucket, buckets)

    def hash(self, key):
        return long(hashlib.md5(key).hexdigest(), 16)

    def ireplicas(self, bucket):
        for r in xrange(self.replicas):
            yield self.hash(
                '%s:%d' % (bucket, r)
            )

    def add_bucket(self, bucket):
        for h in self.ireplicas(bucket):
            if h in self.bucket_map:
                raise ValueError("Bucket %r is already present." % bucket)

            self.bucket_map[h] = bucket
            bisect.insort(self.keys, h)

    def remove_bucket(self, bucket):
        for h in self.ireplicas(bucket):
            # will raise KeyError for nonexistent bucket
            del self.bucket_map[h]
            del self.keys[
                bisect.bisect_left(self.keys, h)
            ]

    def select_bucket(self, key):
        """
        Return a bucket, given a key.
        The bucket replica with a hash value nearest but not less than that of the given key is returned.
        If the hash of the given key is greater than the greatest hash, returns the lowest hashed bucket.
        """
        h = self.hash(
            str(key)
        )
        start = bisect.bisect(self.keys, h)

        if start == len(self.keys):
            start = 0

        return self.bucket_map[
            self.keys[start]
        ]


if __name__ == '__main__':
    from collections import defaultdict

    ring1 = ConsistentHashRing('abcdefghijklm')
    ring2 = ConsistentHashRing('abcdefghijklmno')

    u1 = defaultdict(int)
    u2 = defaultdict(int)

    cnt = [0, 0]

    for key in xrange(4000):
        bucket1 = ring1.select_bucket(key)
        u1[bucket1] += 1

        bucket2 = ring2.select_bucket(key)
        u2[bucket2] += 1

        eq = bucket1 != bucket2
        cnt[eq] += 1

        print '%10d %10s %10s     ' % (key, bucket1, bucket2), ['', 'CHANGES!'][eq]

    print
    print cnt, '\t%.1f%% of keys change its bucket' % (100.*cnt[1]/sum(cnt))

    print
    for k in set(u1) | set(u2):
        v1 = u1.get(k, 0)
        v2 = u2.get(k, 0)
        print '%10s %10d   %-70s    %10d   %-70s' % (
            k,
            v1,
            '*' * (v1/5),
            v2,
            '*' * (v2/5),
        )



from django.utils.crypto import salted_hmac, constant_time_compare
from django.utils.encoding import smart_str


class SecretTokenGenerator(object):
    """
    Utility for protecting the strings (usually) against unauthorized change.
    First call make_token on the protectable and send the result token to
    the wild together with the protectable.
    When it returns, the positive result of check_token tells you the
    protectable was most likely really generated by us.
    Get inspiration on how to make this protection stronger from
    django.contrib.auth.tokens.PasswordResetTokenGenerator
    """
    def make_token(self, protectable):
        p = smart_str(protectable)
        return salted_hmac(self.__class__.__name__, p).hexdigest()[::-2]

    def check_token(self, protectable, token):
        return constant_time_compare(token, self.make_token(protectable))



import os
import sys
import pwd
import grp
import logging.handlers
import signal
import atexit

# For becoming the daemon django.utils.daemonize.become_daemon() is used.
# The additional daemon-stuff utilities follow:


def drop_privileges(user='nobody', group='nogroup'):
    """
    Drop privileges if we're superuser by changing UID/GID to given user/group.
    """
    if os.geteuid() == 0:
        # Remove group privileges
        os.setgroups([])

        if group != 'root':
            try:
                newgid = grp.getgrnam(group).gr_gid
            except KeyError:
                raise RuntimeError("System group '%s' not found." % group)

            os.setgid(newgid)
            os.setegid(newgid)

        if user != 'root':
            try:
                newuid = pwd.getpwnam(user).pw_uid
            except KeyError:
                raise RuntimeError("System user '%s' not found." % user)

            os.setuid(newuid)
            os.seteuid(newuid)


def install_termination_logging_signal_handlers():
    """
    Install the handler for every reconfigurable signal (except some
    ignorables). On signal, the event is logged and process is terminated.
    """
    def sig_handler(signum, unused_frame):
        signames = [n for n, v in signal.__dict__.iteritems()
                    if n.startswith('SIG') and v == signum]
        signame = signames and ' (%s)' % signames[0] or ''
        logging.info("Terminating with signal %d%s." % (signum, signame))
        sys.exit(2)     # calls exit_function

    for s in xrange(100):
        if s not in (signal.SIGCHLD, signal.SIGURG, signal.SIGWINCH):
            try:
                signal.signal(s, sig_handler)
            except:
                pass


def init_syslog(level, process_ident, address='/dev/log', facility='daemon'):
    """
    Set the root logger to be directed to syslog from the given level
    and with given ident string (which will get prepended to every message).
    """
    logging.root.setLevel(level)
    if len(logging.root.handlers) == 0:
        fmt = '%s %%(levelname)s: %%(message)s' % process_ident
        hdlr = logging.handlers.SysLogHandler(address, facility)
        hdlr.setFormatter(logging.Formatter(fmt))
        logging.root.addHandler(hdlr)


def setup_pidfile(pidfile_path):
    """
    Fails if there is something at the given path. Otherwise, writes
    the current PID to file at that path. Also sets the atexit
    handler to automatically remove the file upon exit.

    Note: The process needs to have file removing privileges in the end.
    """
    def exit_function(pidfile_path):
        try:
            logging.debug("Removing PID file %s." % pidfile_path)
            os.unlink(pidfile_path)
        except:
            logging.error("Cannot remove PID file %s." % pidfile_path)

    if os.path.exists(pidfile_path):
        logging.error("Pid file %s exists, aborting daemonization. "
                      "Please assure that I'm not already running." % pidfile_path)
        sys.exit(3)

    atexit.register(exit_function, pidfile_path)
    open(pidfile_path, 'w').write('%d\n' % os.getpid())



from itertools import ifilter


def first(predicate_or_None, iterable, default=None):
    """
    Returns the first item of iterable for which predicate(item) is true.
    If predicate is None, matches the first item that is true.
    Returns value of default in case of no matching items.
    """
    return next(ifilter(predicate_or_None, iterable), default)


def int_or_0(value):
    try:
        return int(value)
    except (ValueError, TypeError):
        return 0


def int_or_None(value):
    try:
        return int(value)
    except (ValueError, TypeError):
        return None


def get_intervals_overlap(a, b):
    """
    >>> get_overlap([0,  5], [10, 20])
    -5
    >>> get_overlap([0,  9], [10, 20])
    -1
    >>> get_overlap([0,  10], [10, 20])
    0
    >>> get_overlap([0,  15], [10, 20])
    5
    >>> get_overlap([10,  15], [10, 20])
    5
    >>> get_overlap([10,  20], [10, 20])
    10
    >>> get_overlap([15,  20], [10, 20])
    5
    >>> get_overlap([15,  15], [10, 20])
    0
    >>> get_overlap([20, 20], [10, 20])
    0
    >>> get_overlap([20, 20], [10, 20])
    0
    >>> get_overlap([30, 40], [10, 20])
    -10
    """
    return min(a[1], b[1]) - max(a[0], b[0])



from collections import defaultdict
import itertools
import operator


def count_by_attr(iterable, by):
    d = defaultdict(int)
    for obj in iterable:
        key = getattr(obj, by)
        d[key] += 1
    return d


def dict_by_attr(iterable, by, get=None, default_factory=None):
    if default_factory is not None:
        d = defaultdict(default_factory)
    else:
        d = {}

    for obj in iterable:
        key = getattr(obj, by)
        if get is not None:
            obj = getattr(obj, get)
        d[key] = obj

    return d


def group_by_attr(iterable, by, get=None):
    d = defaultdict(list)
    for obj in iterable:
        key = getattr(obj, by)
        if get is not None:
            obj = getattr(obj, get)
        d[key].append(obj)
    return d


def iter_compare_dicts(dict1, dict2, only_common_keys=False, comparison_op=operator.ne):
    """
    A generator for comparation of values in the given two dicts.

    Yields the tuples (key, pair of values positively compared).

    By default, the *difference* of values is evaluated using the usual != op, but can be changed
    by passing other comparison_op (a function of two arguments returning True/False).

    For example: operator.eq for equal values, operator.is_not for not identical objects.

    You can also require comparison only over keys existing in both dicts (only_common_keys=True).
    Otherwise, you will get the pair with the Python built-in Ellipsis placed for dict with
    that key missing. (Be sure to test for Ellipsis using the 'is' operator.)

    >>> d1 = dict(a=1, b=2, c=3)
    >>> d2 = dict(a=1, b=20, d=4)
    >>> dict(iter_compare_dicts(d1, d2, only_common_keys=True))
    {'b': (2, 20)}
    >>> dict(iter_compare_dicts(d1, d2, only_common_keys=True, comparison_op=operator.eq))
    {'a': (1, 1)}
    >>> dict(iter_compare_dicts(d1, d2))
    {'c': (3, Ellipsis), 'b': (2, 20), 'd': (Ellipsis, 4)}
    >>> dict(iter_compare_dicts(d1, d2, comparison_op=operator.eq))
    {'a': (1, 1), 'c': (3, Ellipsis), 'd': (Ellipsis, 4)}
    """
    keyset1, keyset2 = set(dict1), set(dict2)

    for key in (keyset1 & keyset2):
        pair = (dict1[key], dict2[key])
        if reduce(comparison_op, pair):
            yield key, pair

    if not only_common_keys:
        for key in (keyset1 - keyset2):
            yield key, (dict1[key], Ellipsis)
        for key in (keyset2 - keyset1):
            yield key, (Ellipsis, dict2[key])


def iter_ibatches(iterable, size):
    """
    http://code.activestate.com/recipes/303279-getting-items-in-batches/

    Generates iterators of elements of fixed size from the source iterable. Does not create batch sequences in memory.
    The source iterable can be of an unknown arbitrary length, does not need to support anything else than iteration.
    itertools.islice provides a size-bounded iterator over the given iterator.

    To know when we're done batching is the tricky part, as islice is happy to continue returning empty iterators
    on source exhaustion. We never want to yield an empty iterator. So we try to consume each batch a bit, which
    possibly raises StopIteration stopping the generator function itself.

    WARNING:
    Each batch must be entirely consumed before proceeding to the next one, otherwise you will get unexpected behaviour!

    >>> for b in iter_ibatches(xrange(55), 10):
    ...     print tuple(b)
    (0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
    (10, 11, 12, 13, 14, 15, 16, 17, 18, 19)
    (20, 21, 22, 23, 24, 25, 26, 27, 28, 29)
    (30, 31, 32, 33, 34, 35, 36, 37, 38, 39)
    (40, 41, 42, 43, 44, 45, 46, 47, 48, 49)
    (50, 51, 52, 53, 54)

    """
    it = iter(iterable)
    while True:
        batch_it = itertools.islice(it, size)
        yield itertools.chain([batch_it.next()], batch_it)



import hashlib

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.utils.encoding import smart_unicode
from django.utils.crypto import salted_hmac, constant_time_compare
from django.contrib.auth.forms import UserChangeForm


CONCURRENCY_EDIT_MESSAGE = _("<br><b>WARNING:</b> Your changes are almost LOST now and can't be "
                             "saved.<br><em>But you still can copy the values out manually "
                             "now.</em><br>Then you can <a href=''>RELOAD</a> and edit again.")


def _clean_object_version(slf):
    form_pk, form_hash = _validate_object_version(slf.cleaned_data)

    # form_pk ---  &&  pk ---    : ok, unsaved object
    # form_pk ---  &&  pk val    : fail as object_version is missing
    # form_pk val  &&  pk ---    : warn about removal
    # form_pk val  &&  pk val    : compare (validation error if mismatch)

    pk = slf.instance.pk

    if form_pk is None:
        if pk is None:
            # newly created object, nothing to check
            return
        slf.concurrency_problem = _("The form is missing the object version check field.")

    elif pk is None:
        slf.concurrency_problem = _("In the mean time, someone <b>removed</b> the item from "
                                    "the database.<br><b>Submit the form again</b> in case "
                                    "you are sure to create it.")
        slf.data = slf.data.copy()    # make the data dict mutable first
        slf.data['object_version'] = ''

    elif form_pk != unicode(pk):
        raise RuntimeError(_("Found not matching primary key in object_version field."))

    if not slf.concurrency_problem:
        db_obj = slf._meta.model.objects.get(pk=pk)
        db_hash = _gen_object_hash(db_obj)
        if form_hash != db_hash:
            slf.concurrency_problem = \
                _("In the mean time, someone <b>changed</b> this item in the database.") \
                + ' ' + unicode(slf._CONCURRENCY_EDIT_MESSAGE)

    if slf.concurrency_problem:
        errors = slf._errors.setdefault(forms.forms.NON_FIELD_ERRORS,
                                        forms.util.ErrorList())
        errors.append(mark_safe(slf.concurrency_problem))


class ConcurrencyProtectionModelForm(forms.ModelForm):
    """
    ModelForm based on this protects the object from concurrent user updates.
    When the form is displayed for edition, the hash of its values is created.
    During cleaning, the hash from the current object in database is created
    again and compared. The mismatch reveals that the object has been changed
    in the meantime -- then the validation error is raised.

    Example::

        class MyModelForm(ConcurrencyProtectionModelForm):
            class Meta:
                model = MyModel

    Use this also in your ModelAdmin descendants::

        class QuestionSetAdmin(admin.ModelAdmin):
            form = ConcurrencyProtectionModelForm

    This way concurrent updates will be watched from both your UI and django.contrib.admin.
    """
    object_version = forms.CharField(required=False, widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        super(ConcurrencyProtectionModelForm, self). __init__(*args, **kwargs)
        self.concurrency_problem = None
        self.fields['object_version'].initial = _gen_object_version(self.instance)

    clean_object_version = _clean_object_version

    _CONCURRENCY_EDIT_MESSAGE = CONCURRENCY_EDIT_MESSAGE


# DEPRECATED -- it does not work in Django 1.6.
#               Use 'form = ConcurrencyProtectionModelForm' instead.
#
# class ConcurrencyProtectionModelAdmin(admin.ModelAdmin):
#     """
#
#     You may want to avoid the situations when the admin user overwrites
#     the changes of anybody else. In that case, inherit your model admins from
#     this class instead of the usual admin.ModelAdmin.
#     The admin form will be monkey patched with the protection field.
#     """
#     def get_form(self, request, obj=None, **kwargs):
#         form = super(ConcurrencyProtectionModelAdmin, self).get_form(request, obj, **kwargs)
#
#         if obj is not None:
#             form.concurrency_problem = None
#             field = forms.CharField(initial=_gen_object_version(obj), required=False)
#             form.base_fields['object_version'] = field
#             form.clean_object_version = _clean_object_version
#         return form


class ConcurrencyProtectionUserChangeForm(UserChangeForm):
    """
    UserAdmin can be re-registered like this::

        from django.contrib.auth.admin import UserAdmin as DjangoUserAdmin
        from useful.django.concurrency import ConcurrencyProtectionUserChangeForm

        class UserAdmin(DjangoUserAdmin):
            form = ConcurrencyProtectionUserChangeForm

            fieldsets = (
                ...,
                (None, {'fields': ('object_version',)}),
            )

        admin.site.unregister(User)
        admin.site.register(User, UserAdmin)
    """
    object_version = forms.CharField(required=False, widget=forms.HiddenInput)

    def __init__(self, *args, **kwargs):
        super(ConcurrencyProtectionUserChangeForm, self). __init__(*args, **kwargs)
        self.concurrency_problem = None
        self.fields['object_version'].initial = _gen_object_version(self.instance)

    clean_object_version = _clean_object_version

    _CONCURRENCY_EDIT_MESSAGE = CONCURRENCY_EDIT_MESSAGE


def _gen_hmac(pk, hash_):
    pk = smart_unicode(pk).encode('UTF-8')
    hash_ = smart_unicode(hash_).encode('UTF-8')
    return salted_hmac('concurrency prot', '%s %s' % (pk, hash_)).hexdigest()


def _gen_object_hash(obj):
    def get_obj_data():
        for k, v in sorted(forms.models.model_to_dict(obj).items()):
            yield u'%s=%s' % (smart_unicode(k), smart_unicode(v))

    data = u' '.join(get_obj_data()).encode('UTF-8')
    return hashlib.sha256(data).hexdigest()


def _gen_object_version(obj):
    """
    Get protected hash of object values.
    """
    # Another approach could be:
    # return obj.modified.strftime('%s.%f')
    if obj.pk is None:
        return ''

    hash_ = _gen_object_hash(obj)
    return u'%s-%s-%s' % (obj.pk, hash_, _gen_hmac(obj.pk, hash_))


def _validate_object_version(datadict):
    form_pk, form_hash = None, None
    form_version = datadict.get('object_version', '')
    if form_version:
        try:
            form_pk, form_hash, form_hmac = form_version.split('-', 2)
            if not form_pk or not form_hash or not form_hmac:
                raise ValueError
        except ValueError:
            raise forms.ValidationError("Bad object_version format.")

        if not constant_time_compare(form_hmac, _gen_hmac(form_pk, form_hash)):
            raise forms.ValidationError(_("Version protection tampered."))

    return form_pk, form_hash



from django.core import urlresolvers
from django.core.exceptions import PermissionDenied
from django.utils.functional import memoize

# This module requires that you use useful.django.urlpatterns.UrlPatterns
# to decorate your views.

_all_callbacks = {}     # caches the callbacks dicts per URLconf


def get_all_callbacks(urlconf):
    """
    Gets the dict translating the view names to view callables for the entire
    given URLconf. Does not allow duplicate view names. Views in non-empty
    namespaces are prefixed with the namespace(s).
    """
    callbacks = {}

    def add_callbacks(resolver, namespace):
        for pattern in resolver.url_patterns:
            if isinstance(pattern, urlresolvers.RegexURLResolver):
                ns = namespace
                if pattern.namespace:
                    ns += (':' if ns else '') + pattern.namespace
                add_callbacks(pattern, ns)

            elif pattern.name is not None:
                ns_name = namespace + (':' if namespace else '') + pattern.name
                prev_callback = callbacks.get(ns_name)
                if prev_callback:
                    if getattr(pattern.callback, 'can_url_perms', None) != \
                       getattr(prev_callback, 'can_url_perms', None):
                        raise RuntimeError("Found multiple URL patterns named %r with different "
                                           "perms. If you want to decorate single view multiple "
                                           "times with urlpatterns.url, add perms= option to the "
                                           "last decorator only, it will apply to all." % ns_name)
                callbacks[ns_name] = pattern.callback

    add_callbacks(urlresolvers.get_resolver(urlconf), '')
    return callbacks
get_all_callbacks = memoize(get_all_callbacks, _all_callbacks, 1)


def can_url(user, view):
    """
    Tests whether the given user would have access to the view. The view can
    be a callable, importable text path or a view name, possibly with the
    namespace prefix ('namespace:view_name'). The view function must be
    decorated with the can_url_func (that's what UrlPatterns class does).
    """
    view = urlresolvers.get_callable(view, True)

    if not callable(view):
        callbacks = get_all_callbacks(urlresolvers.get_urlconf())
        if view not in callbacks:
            raise urlresolvers.NoReverseMatch("Reverse for '%s' not found." % view)

        view = callbacks[view]

    if not hasattr(view, 'can_url_func'):
        raise urlresolvers.NoReverseMatch("Reverse for '%s' is not decorated with permissions." % view)

    try:
        return view.can_url_func(user)
    except PermissionDenied:
        return False


class CanUrl(object):
    """
    Helper wrapper for can_url in templates. Allows for short and readable
    testing, whether the given view is accessible by the current user.
    """
    def __init__(self, user):
        self.user = user

    def __getitem__(self, view_name):
        """
        Support for::
            {% if can_url.view_name %} ...
        """
        return can_url(self.user, view_name)

    def __contains__(self, view_name):
        """
        Support for::
            {% if 'namespace:view_name' in can_url %} ...
        """
        return can_url(self.user, view_name)


def can_url_processor(request):
    """
    A context processor adding the template helper for querying view-level
    permissions like this::

        ### settings.py

        TEMPLATE_CONTEXT_PROCESSORS += (
            ...
            'useful.django.can_url.can_url_processor',
        )

        ### in some template.html

        {% if can_url.view_name %}
            <a href="{% url 'view_name' %}">...</a>
        {% endif %}
    """
    if hasattr(request, 'user'):
        user = request.user
    else:
        from django.contrib.auth.models import AnonymousUser
        user = AnonymousUser()

    return {'can_url': CanUrl(user)}



from collections import defaultdict

from django.db import connection
from django.db.models.manager import Manager
from django.db.models.query import QuerySet


def _get_queryset(klass):
    """
    Returns a QuerySet from a Model, Manager, or QuerySet.

    NoDRY: Function copied from __init__ in django.shortcuts to assure stability.
    """
    if isinstance(klass, QuerySet):
        return klass
    elif isinstance(klass, Manager):
        manager = klass
    else:
        manager = klass._default_manager
    return manager.all()


def get_object_or_none(klass, *args, **kwargs):
    """
    Uses get() to return an object, or None if the object does not exist.

    klass may be a Model, Manager, or QuerySet object. All other passed
    arguments and keyword arguments are used in the get() query.

    Note: Like with get(), an MultipleObjectsReturned will be raised if more than one
    object is found.
    """
    queryset = _get_queryset(klass)
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return None


def get_object_or_new(klass, *args, **kwargs):
    """
    Uses get() to return an object. If it does not exist, a new UNSAVED
    object of the given model is returned.

    klass may be a Model, Manager, or QuerySet object. All other passed
    arguments and keyword arguments are used in the get() query.

    Note: Like with get(), an MultipleObjectsReturned will be raised if more than one
    object is found.
    """
    queryset = _get_queryset(klass)
    try:
        return queryset.get(*args, **kwargs)
    except queryset.model.DoesNotExist:
        return queryset.model(*args, **kwargs)


def prefetch_m2m(m2m_field):
    """
    Loads all objects from the target table of the given ManyToManyField,
    then loads the entire intermediary join table.
    Returns the lookup where key is the object id and value is the list
    of target objects.

    Usage example to get comma-separated list of user's groups in template::

        groups_m2m = prefetch_m2m(User.groups)

          {{ groups_m2m|get:user.id|default_if_none:""|join:", " }}
    """
    f = m2m_field.field
    tgt_objs = dict((o.pk, o) for o in f.rel.to.objects.all())

    cursor = connection.cursor()
    cursor.execute("SELECT %s, %s FROM %s" % (f.m2m_column_name(),
                                              f.m2m_reverse_name(),
                                              f.m2m_db_table()))
    lookup = defaultdict(list)
    for src, tgt in cursor.fetchall():
        lookup[src].append(tgt_objs[tgt])

    return lookup


def get_values_map(klass, by, *fields):
    """
    Creates the Python set or dict from data in klass.
    klass may be a Model, Manager, or QuerySet object.
    If no fields are given, the result is the set of all distinct values of 'by'.
    If fields is a single field, the result is dict by->field.
    Otherwise the key of dict is still 'by', the value is tuple of fields.
    """
    queryset = _get_queryset(klass)

    iterator = queryset.values_list(by, *fields).order_by().iterator()

    if len(fields) > 1:
        return dict((tup[0], tup[1:]) for tup in iterator)

    elif len(fields) == 1:
        return dict(iterator)

    else:
        return set(queryset.values_list(by, flat=True).distinct().order_by().iterator())



import httplib
import socket
import urllib2

# Python 2.6 required for the ssl module.
import ssl  # @UnresolvedImport

# Based on http://www.muchtooscrawled.com/2010/03/https-certificate-verification-in-python-with-urllib2/     @IgnorePep8

# Example usage:
# response = urlopen('https://api-3t.sandbox.paypal.com/nvp',
#    '/path/to/VeriSign-Class3_Public_Primary_Certification_Authority_G2.pem',
#    'PayPal').read()


class VerifiedHTTPSConnection(httplib.HTTPSConnection):
    CA_FILE_PATH = None
    REQUIRED_ORGANIZATION_PATTERN = None

    def match_organization(self, cert):
        if self.REQUIRED_ORGANIZATION_PATTERN is None:
            return True

        for rdn in cert['subject']:
            for k, v in rdn:
                if k == 'organizationName':
                    if self.REQUIRED_ORGANIZATION_PATTERN in v:
                        return True

        return False

    def connect(self):
        # overrides the version in httplib so that we do
        #    certificate verification
        sock = socket.create_connection((self.host, self.port), self.timeout)
        if self._tunnel_host:
            self.sock = sock
            self._tunnel()

        # wrap the socket using verification with the root
        #    certs in trusted_root_certs
        self.sock = ssl.wrap_socket(sock,
                                    self.key_file,
                                    self.cert_file,
                                    cert_reqs=ssl.CERT_REQUIRED,
                                    ca_certs=self.CA_FILE_PATH)
        cert = self.sock.getpeercert()
        if not self.match_organization(cert):
            raise urllib2.URLError("Server cert issued for wrong organization")


# wraps https connections with ssl certificate verification
class VerifiedHTTPSHandler(urllib2.HTTPSHandler):
    def __init__(self, connection_class=VerifiedHTTPSConnection):
        self.specialized_conn_class = connection_class
        urllib2.HTTPSHandler.__init__(self)

    def https_open(self, req):
        return self.do_open(self.specialized_conn_class, req)


def urlopen(url, ca_file_path, org_name_pattern=None, data=None):
    """
    urllib2.urlopen extended to verify whether the HTTPS the server certificate
    was issued by the given CA. The CA certificate file path must
    be provided.
    If the org_name_pattern is given, the organizationName of the server
    certificate is searched for the substring.
    """
    # Patches the class to set the required info.
    VerifiedHTTPSConnection.CA_FILE_PATH = ca_file_path
    VerifiedHTTPSConnection.REQUIRED_ORGANIZATION_PATTERN = org_name_pattern
    https_handler = VerifiedHTTPSHandler()
    url_opener = urllib2.build_opener(https_handler)
    return url_opener.open(url, data)



class Choices:
    """
    Encapsulates the choices definition and conversion, praises the DRY.

    Allows for using custom symbols as individual choices, bundles them with i18n-enabled labels
    and stores choices in a way that usually fits more for processing, such as small integers.

    Example::

        class Payment(models.Model):
            STATUS = Choices(initial      = (100, _("Awaiting Payment")),
                             success      = (200, _("Paid OK")),
                             nok_failure  = (300, _("Payment Failed")))
            ...
            status = models.SmallIntegerField(...,
                                              choices=STATUS.choices(),
                                              default=STATUS.initial)
            ...

    Second choice can be referred as Payment.STATUS.success throughout
    the code, equals to 200.
    """
    def __init__(self, **defs):
        """
        Takes the dictionary, where keys are identifiers and values
        are tuples (db id, human value).
        """
        if [True for k in defs if k.startswith('_')]:
            raise RuntimeError("Programming error: Names starting with "
                               "underscore are not allowed in Choices.")
        self.defs = defs
        self.names = dict((v[0], k) for k, v in defs.iteritems())
        self.texts = dict(defs.values())
        if len(set(self.names.keys())) != len(defs):
            raise RuntimeError("Programming error: Duplicate choices defined!")

    def __add__(self, other):
        """
        Support for Choices(...) + Choices(...)
        """
        defs = self.defs.copy()
        defs.update(other.defs)
        return Choices(**defs)

    def choices(self):
        ch = self.defs.values()
        ch.sort(lambda x, y: cmp(x[0], y[0]))
        return ch

    def get(self, name, default=None):
        """
        Example: Choices(...).get('success') -> 200
        """
        return self.defs.get(name, (default,))[0]

    def __getattr__(self, name):
        """
        Example: Choices(...).success -> 200
        """
        if name.startswith('_'):
            raise AttributeError("No attribute %s in Choices class" % name)
        return self.defs[name][0]

    def name_of(self, what):
        """
        Example: name_of(200) -> 'success'
        """
        return self.names[what]

    def text_of(self, what):
        """
        Example: text_of(200) -> "Paid OK"    # note: a gettext string
        """
        return self.texts[what]

    def tuples(self):
        """
        Generates tuples (db id, short name, human name).
        """
        for k, v in self.defs.items():
            # Enforcing i18n by unicode()
            yield v[0], k, unicode(v[1])

    def select(self, keys):
        """
        Returns new Choice object with subset of choices.
        """
        if not isinstance(keys, (list, tuple)):
            keys = [keys]
        return Choices(**dict((k, self.defs[k])
                       for k in keys if k in self.defs))

    def sql_case_ids_to_names(self, field_name):
        whens = ["WHEN %d THEN '%s'" % (v[0], k) for k, v in self.defs.items()]
        return "CASE %s %s ELSE '?' END" % (field_name, ' '.join(whens))


def create_choices_tests(klass):
    """
    Decorator adds is_<choiceset name>_<choice> properties to class
    for all Choices instances inside.
    For example::

        @create_choices_tests
        class Payment:
             # see the docstring of Choices class for details

    so this is possible::

        p = Payment(status=Payment.STATUS.initial)
        if p.is_status_initial:
            (it's true)
    """
    for chname, attr in klass.__dict__.items():
        if isinstance(attr, Choices):
            for name in attr.defs:
                fn = 'is_%s_%s' % (chname.lower(), name)
                if hasattr(klass, fn):
                    raise RuntimeError("create_choices_tests: Attribute %s.%s "
                                       "already exists." % (klass.__name__, fn))
                f = eval("lambda slf: slf.%s == slf.%s.%s" % (chname.lower(),
                                                              chname, name))
                setattr(klass, fn, property(f))
    return klass



from os.path import dirname, join
from setuptools import setup, find_packages

import useful

with open(join(dirname(__file__), 'README.rst')) as readme_file:
    long_description = readme_file.read()

setup(
    name='useful',
    version=useful.__versionstr__,
    packages=find_packages(),
    package_data={'useful': ['django/locale/cs/LC_MESSAGES/django.*']},
    description="Everyday use utilities for writing Python code or Django projects.",
    long_description=long_description,
    author="Vlada Macek",
    author_email="macek@sandbox.cz",
    url="https://github.com/tuttle/python-useful",
    license='BSD License',

    zip_safe=False,

    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Topic :: Software Development :: Libraries",
    ],
)



import re

from django.conf import settings
from django.utils.translation import get_language


def get_safe_langcode():
    """
    Returns two-letter code of the active language.
    Is always one of the supported languages. 'en' is expected to exist!
    """
    lang = get_language()[:2].lower()
    return lang if lang in dict(settings.LANGUAGES) else 'en'


def add_language_getter_for(*field_names):
    """
    Class decorator. For each field given as argument, adds a reading property
    with the same name to the wrapped class. When that property is accessed
    in the instance the value of the real field <field_name>_<lang> is
    returned, where lang is the current Django language two-letter code.
    In case the attribute for the current language does not exist
    or its content is empty, the english value is returned as a fallback.
    """
    def get_getter_for(fld):
        def getter(slf):
            value = getattr(slf, fld + '_' + get_safe_langcode(), None)
            if not value:
                value = getattr(slf, fld + '_en')
            return value
        return getter

    def add_language_getter(klass):
        for fld in field_names:
            if not re.match('^[_a-zA-Z][_a-zA-Z0-9]*$', fld):
                raise RuntimeError("Field name %r has not a valid identifier "
                                   "syntax." % fld)
            if not hasattr(klass, '_language_getters'):
                klass._language_getters = set()
            if fld not in klass._language_getters and hasattr(klass, fld):
                raise RuntimeError("add_language_getter: Attribute %s.%s "
                                   "already exists." % (klass.__name__, fld))

            setattr(klass, fld, property(get_getter_for(fld)))
            klass._language_getters.add(fld)
        return klass
    return add_language_getter



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class SetRemoteAddrMiddleware(object):
    """
    Request middleware that sets REMOTE_ADDR based on request.META variable
    pointed by settings.REAL_IP_META_VARIABLE_NAME.

    This is intentionally a required setting, although if it is None,
    the feature is disabled. That's useful for local development.

    In production use, for example if nginx is your reverse proxy, you can
    configure it like this::

        proxy_set_header   X-Real-IP        $remote_addr;

    and if you set Django like this::

        REAL_IP_META_VARIABLE_NAME = 'HTTP_X_REAL_IP'

    then will get the real remote IP address to REMOTE_ADDR.
    """
    def process_request(self, request):
        try:
            real_ip_varname = settings.REAL_IP_META_VARIABLE_NAME
        except AttributeError:
            raise ImproperlyConfigured("%s: Missing the required setting "
                                       "REAL_IP_META_VARIABLE_NAME."
                                       % self.__class__.__name__)

        if real_ip_varname is not None:
            request.META['REMOTE_ADDR'] = request.META[real_ip_varname]
